@@ -59,14 +59,17 @@ sub find_tarball {
     my ($chroot)       = @_;
     my $tarball        = undef;
     my $xdg_cache_home = $self->get_conf('HOME') . "/.cache/sbuild";
-    if (defined($ENV{'XDG_CACHE_HOME'})) {
+    if (length($ENV{'XDG_CACHE_HOME'})) {
         $xdg_cache_home = $ENV{'XDG_CACHE_HOME'} . '/sbuild';
     }
     if (opendir my $dh, $xdg_cache_home) {
         while (defined(my $file = readdir $dh)) {
             next if $file eq '.' || $file eq '..';
             my $path = "$xdg_cache_home/$file";
-            next if -z $path;
+            if (-z $path) {
+                print STDERR "I: ignoring $path (zero size)\n";
+                next;
+            }
             if ($file =~ /^$chroot\.t.+$/) {
                 $tarball = $path;
                 last;
@@ -85,45 +88,53 @@ sub chroot_tarball_if_too_old {
         return undef;
     }
     my $tarball = $self->find_tarball($chroot);
-    if (defined($tarball)) {
-        my $max_age = $self->get_conf('UNSHARE_MMDEBSTRAP_MAX_AGE');
-        if (!-e $tarball) {
-            print STDERR "I: Chroot Tarball $tarball does not exist yet\n";
-            if ($max_age < 0) {
-                print STDERR
-                  "I: Not updating it due to negative maximum age\n";
-                return undef;
-            }
-            return $tarball;
+    if (!defined($tarball)) {
+        # We end up here if the user added the --chroot option but there
+        # was no associated tarball found. Create a new tarball using the
+        # chroot name.
+        my $xdg_cache_home = $self->get_conf('HOME') . "/.cache/sbuild";
+        if (length($ENV{'XDG_CACHE_HOME'})) {
+            $xdg_cache_home = $ENV{'XDG_CACHE_HOME'} . '/sbuild';
         }
-        # negative max-age indicates to never update
-        # if an existing tarball is too young, don't update
-        my $age     = time - (stat($tarball))[9];
-        if ($max_age >= 0 && $age >= $max_age) {
-            print STDERR "I: Existing chroot tarball is too old ("
-              . (
-                sprintf '%.2f >= %.2f',
-                ($age / 60 / 60 / 24),
-                ($max_age / 60 / 60 / 24)) . " days):\n";
-            print STDERR ("I: Change the maximum age by setting "
-                  . "\$unshare_mmdebstrap_max_age (in seconds) in your "
-                  . "~/.sbuildrc or disable it by setting it to a "
-                  . "negative value.\n");
-            return $tarball;
+        $tarball = "$xdg_cache_home/$chroot.tar";
+    }
+
+    my $max_age = $self->get_conf('UNSHARE_MMDEBSTRAP_MAX_AGE');
+    if (!-e $tarball) {
+        print STDERR "I: Chroot Tarball $tarball does not exist yet\n";
+        if ($max_age < 0) {
+            print STDERR "I: Not updating it due to negative maximum age\n";
+            return undef;
         }
-        return undef;
+        return $tarball;
     }
-    # this should never happen because $chroot was computed by
-    # ChrootInfo.pm and should thus be an already existing path
-    print STDERR ("W: Existing chroot tarball was not "
-          . "found even though it should've been.\n");
-    print STDERR ("W: THIS SHOULD NEVER HAPPEN."
-          . "Please file a bug if you are seeing this.\n");
-    my $xdg_cache_home = $self->get_conf('HOME') . "/.cache/sbuild";
-    if (defined($ENV{'XDG_CACHE_HOME'})) {
-        $xdg_cache_home = $ENV{'XDG_CACHE_HOME'} . '/sbuild';
+    # negative max-age indicates to never update
+    # if an existing tarball is too young, don't update
+    my $age = time - (stat($tarball))[9];
+    if ($max_age >= 0 && $age >= $max_age) {
+        my $config_path = '~/.config/sbuild/config.pl';
+        if (length($ENV{'XDG_CONFIG_HOME'})) {
+            $config_path = $ENV{'XDG_CONFIG_HOME'} . '/sbuild/config.pl';
+        }
+        print STDERR "I: Existing chroot tarball is too old ("
+          . (
+            sprintf '%.2f >= %.2f',
+            ($age / 60 / 60 / 24),
+            ($max_age / 60 / 60 / 24)) . " days):\n";
+        print STDERR ("I: Change the maximum age by setting "
+              . "\$unshare_mmdebstrap_max_age (in seconds)\n"
+              . "I: in your $config_path or disable it by "
+              . "setting it to a negative value.\n");
+        return $tarball;
     }
-    return "$xdg_cache_home/$chroot.tar";
+    return undef;
+}
+
+# from List/MoreUtils/PP.pm
+sub natatime ($@) {
+    my $n    = shift;
+    my @list = @_;
+    return sub { return splice @list, 0, $n; }
 }
 
 sub chroot_auto_create {
@@ -134,27 +145,34 @@ sub chroot_auto_create {
     my $arch    = $self->get_conf('BUILD_ARCH');
 
     my $xdg_cache_home = $self->get_conf('HOME') . "/.cache/sbuild";
-    if (defined($ENV{'XDG_CACHE_HOME'})) {
+    if (length($ENV{'XDG_CACHE_HOME'})) {
         $xdg_cache_home = $ENV{'XDG_CACHE_HOME'} . '/sbuild';
     }
 
-    if ($dist eq "UNRELEASED") {
-        print STDERR "W: translating UNRELEASED to unstable";
-        $dist = "unstable";
-    }
-
     # mmdebstrap chooses Essential:yes packages from the given
     # distribution, so for experimental or backports, we have to
     # pass a different dist (unstable and stable, respectively). We could
     # also pass an empty dist string but then mmdebstrap cannot anymore
     # choose the stable mirrors for us.
     my $basedist = $dist;
-    if ($dist eq "experimental" or $dist eq "rc-buggy") {
-        print STDERR "I: choosing unstable as base distribution for $dist\n";
-        $basedist = "unstable";
-    } elsif ($dist =~ m/^(.*)-backports$/) {
-        print STDERR "I: choosing $1 as base distribution for $dist\n";
-        $basedist = $1;
+    if (scalar(@{ $self->get_conf('UNSHARE_MMDEBSTRAP_DISTRO_MANGLE') })) {
+        if (
+            scalar(@{ $self->get_conf('UNSHARE_MMDEBSTRAP_DISTRO_MANGLE') })
+            % 2 != 0) {
+            print STDERR
+              "W: length of UNSHARE_MMDEBSTRAP_DISTRO_MANGLE is uneven\n";
+        }
+        my $it = natatime 2,
+          @{ $self->get_conf('UNSHARE_MMDEBSTRAP_DISTRO_MANGLE') };
+        while (my ($k, $v) = $it->()) {
+            if ($dist !~ m/$k/) {
+                next;
+            }
+            ($basedist = $dist) =~ s/$k/$v/;
+            print STDERR ("I: Applied base distribution name mangle rule "
+                  . "s/$k/$v/ turning \"$dist\" into \"$basedist\"\n");
+            last;
+        }
     }
 
     my @commonargs = (
@@ -196,14 +214,15 @@ sub chroot_auto_create {
             $tarball = "$xdg_cache_home/$dist-$arch.tar";
             $chroot  = "$xdg_cache_home/$dist-$arch.tar";
             $self->set('Chroot ID', $chroot);
-            print STDERR ("I: Creating new chroot tarball:\n"
-                  . join(" ", (@commonargs, $tarball, @{$extraargs}))
-                  . "\n");
         }
+        print STDERR ("I: Creating new chroot tarball:\n"
+              . join(" ", (@commonargs, $tarball, @{$extraargs}))
+              . "\n");
 
         my $exit_code = system @commonargs, $tarball, @{$extraargs};
         if ($exit_code != 0) {
             print STDERR "mmdebstrap failed\n";
+            unlink $tarball;
             return undef;
         }
 
@@ -300,12 +319,17 @@ sub chroot_auto_create {
           )
           . "\n"
     );
+    my $config_path = '~/.config/sbuild/config.pl';
+    if (length($ENV{'XDG_CONFIG_HOME'})) {
+        $config_path = $ENV{'XDG_CONFIG_HOME'} . '/sbuild/config.pl';
+    }
     print STDERR ("I:  - or let sbuild take care of this via the setting "
           . "UNSHARE_MMDEBSTRAP_KEEP_TARBALL by adding "
-          . "'\$unshare_mmdebstrap_keep_tarball = 1;' to your ~/.sbuildrc.\n");
+          . "'\$unshare_mmdebstrap_keep_tarball = 1;' to your $config_path.\n"
+    );
     print STDERR ("I:  - or completely disable this behaviour via the setting "
           . "UNSHARE_MMDEBSTRAP_AUTO_CREATE by adding "
-          . "'\$unshare_mmdebstrap_auto_create = 0;' to your ~/.sbuildrc.\n");
+          . "'\$unshare_mmdebstrap_auto_create = 0;' to your $config_path.\n");
     print STDERR (
             "I: Refer to UNSHARE_MMDEBSTRAP_KEEP_TARBALL in sbuild.conf(5) "
           . "for more information\n");
@@ -400,7 +424,7 @@ sub begin_session {
         $tarball = $self->find_tarball($chroot);
         if (!defined($tarball)) {
             my $xdg_cache_home = $self->get_conf('HOME') . "/.cache/sbuild";
-            if (defined($ENV{'XDG_CACHE_HOME'})) {
+            if (length($ENV{'XDG_CACHE_HOME'})) {
                 $xdg_cache_home = $ENV{'XDG_CACHE_HOME'} . '/sbuild';
             }
 
